@@ -31,6 +31,10 @@
 
 #include "config.h"
 
+#include <time.h>
+#include <math.h>
+#include <sys/time.h>
+
 #include "listview_curses.hh"
 
 using namespace std;
@@ -44,7 +48,10 @@ listview_curses::listview_curses()
       lv_left(0),
       lv_height(0),
       lv_needs_update(true),
-      lv_show_scrollbar(true)
+      lv_show_scrollbar(true),
+      lv_word_wrap(false),
+      lv_mouse_y(-1),
+      lv_mouse_mode(LV_MODE_NONE)
 { }
 
 listview_curses::~listview_curses()
@@ -96,12 +103,12 @@ bool listview_curses::handle_key(int ch)
     case 'b':
     case KEY_BACKSPACE:
     case KEY_PPAGE:
-        this->shift_top(-height);
+        this->shift_top(-(this->rows_available(this->lv_top, RD_UP) - vis_line_t(1)));
         break;
 
     case ' ':
     case KEY_NPAGE:
-        this->shift_top(height);
+        this->shift_top(this->rows_available(this->lv_top, RD_DOWN) - vis_line_t(1));
         break;
 
     case KEY_HOME:
@@ -109,10 +116,9 @@ bool listview_curses::handle_key(int ch)
         break;
 
     case KEY_END: {
-            vis_line_t tail_bottom(this->get_inner_height() - height + 1);
             vis_line_t last_line(this->get_inner_height() - 1);
+            vis_line_t tail_bottom(this->get_top_for_last_row());
 
-            tail_bottom = max(vis_line_t(0), tail_bottom);
             if (this->get_top() == last_line)
                 this->set_top(tail_bottom);
             else if (tail_bottom <= this->get_top())
@@ -126,7 +132,7 @@ bool listview_curses::handle_key(int ch)
         {
             double tenth = ((double)this->get_inner_height()) / 10.0;
 
-            this->shift_top(vis_line_t(tenth));
+            this->shift_top(vis_line_t((int)tenth));
         }
         break;
 
@@ -134,7 +140,7 @@ bool listview_curses::handle_key(int ch)
         {
             double tenth = ((double)this->get_inner_height()) / 10.0;
 
-            this->shift_top(vis_line_t(-tenth));
+            this->shift_top(vis_line_t((int)-tenth));
         }
         break;
 
@@ -149,11 +155,11 @@ bool listview_curses::handle_key(int ch)
 void listview_curses::do_update(void)
 {
     if (this->lv_window != NULL && this->lv_needs_update) {
-        vis_line_t        y(this->lv_y), height, bottom, lines, row;
+        vis_line_t        y(this->lv_y), height, bottom, row;
         attr_line_t       overlay_line;
         vis_line_t        overlay_height(0);
         struct line_range lr;
-        unsigned long     width;
+        unsigned long     width, wrap_width;
         size_t            row_count;
 
         if (this->lv_overlay_source != NULL) {
@@ -162,19 +168,14 @@ void listview_curses::do_update(void)
         }
 
         this->get_dimensions(height, width);
-        lr.lr_start = this->lv_left;
-        lr.lr_end   = this->lv_left + width;
+        wrap_width = width - (this->lv_word_wrap ? 1 : 0);
 
         row_count = this->get_inner_height();
-        if (this->lv_top >= (int)row_count) {
-            this->lv_top = max(vis_line_t(0), vis_line_t(row_count) - height);
-        }
-
         row   = this->lv_top;
-        lines = min(height - overlay_height,
-                    vis_line_t(row_count) - this->lv_top);
         bottom = y + height;
-        for (; y < bottom; ++y) {
+        while (y < bottom) {
+            lr.lr_start = this->lv_left;
+            lr.lr_end   = this->lv_left + wrap_width;
             if (this->lv_overlay_source != NULL &&
                 this->lv_overlay_source->list_value_for_overlay(
                     *this,
@@ -182,29 +183,39 @@ void listview_curses::do_update(void)
                     overlay_line)) {
                 this->mvwattrline(this->lv_window, y, 0, overlay_line, lr);
                 overlay_line.clear();
+                ++y;
             }
-            else if (lines > 0) {
+            else if (row < (int)row_count) {
                 attr_line_t al;
 
                 this->lv_source->listview_value_for_row(*this, row, al);
-                this->mvwattrline(this->lv_window, y, 0, al, lr);
-                --lines;
+                do {
+                    this->mvwattrline(this->lv_window, y, 0, al, lr);
+                    if (this->lv_word_wrap) {
+                        wmove(this->lv_window, y, wrap_width);
+                        wclrtoeol(this->lv_window);
+                    }
+                    lr.lr_start += wrap_width;
+                    lr.lr_end += wrap_width;
+                    ++y;
+                } while (this->lv_word_wrap && y < bottom && lr.lr_start < (int)al.length());
                 ++row;
             }
             else {
                 wmove(this->lv_window, y, 0);
                 wclrtoeol(this->lv_window);
+                ++y;
             }
         }
 
         if (this->lv_show_scrollbar) {
             double progress = 1.0;
             double coverage = 1.0;
+            vis_line_t lines;
 
             if (this->get_inner_height() > 0) {
-                progress = (double)this->lv_top /
-                           (double)this->get_inner_height();
-                coverage = (double)height / (double)this->get_inner_height();
+                progress = (double)this->lv_top / (double)row_count;
+                coverage = (double)height / (double)row_count;
             }
 
             y = vis_line_t(this->lv_y) +
@@ -212,12 +223,12 @@ void listview_curses::do_update(void)
             lines = y + min(height, vis_line_t(
                                 (int)(coverage * (double)height)));
             for (; y <= lines; ++y) {
-                char buffer;
-
-                mvwinnstr(this->lv_window, y, width - 1, &buffer, 1);
-                wattron(this->lv_window, A_REVERSE);
-                mvwaddnstr(this->lv_window, y, width - 1, &buffer, 1);
-                wattroff(this->lv_window, A_REVERSE);
+                mvwchgat(this->lv_window,
+                         y, width - 1,
+                         1,
+                         A_REVERSE, view_colors::ansi_color_pair(
+                            COLOR_WHITE, COLOR_BLACK),
+                         NULL);
             }
             wmove(this->lv_window, this->lv_y + height - 1, 0);
         }
@@ -225,3 +236,104 @@ void listview_curses::do_update(void)
         this->lv_needs_update = false;
     }
 }
+
+static int scroll_polarity(mouse_button_t button)
+{
+    return button == BUTTON_SCROLL_UP ? -1 : 1;
+}
+
+bool listview_curses::handle_mouse(mouse_event &me)
+{
+    vis_line_t inner_height, height;
+    struct timeval diff;
+    unsigned long width;
+
+    timersub(&me.me_time, &this->lv_mouse_time, &diff);
+    this->get_dimensions(height, width);
+    inner_height = this->get_inner_height();
+
+    switch (me.me_button) {
+    case BUTTON_SCROLL_UP:
+    case BUTTON_SCROLL_DOWN:
+        if (diff.tv_sec > 0 || diff.tv_usec > 80000) {
+            this->lv_scroll_accel = 1;
+            this->lv_scroll_velo = 0;
+        }
+        else {
+            this->lv_scroll_accel += 3;
+        }
+        this->lv_scroll_velo += this->lv_scroll_accel;
+
+        this->shift_top(vis_line_t(scroll_polarity(me.me_button) *
+                                   this->lv_scroll_velo),
+                        true);
+        break;
+    default:
+        break;
+    }
+    this->lv_mouse_time = me.me_time;
+
+    if (me.me_button != BUTTON_LEFT ||
+        inner_height == 0 ||
+        (this->lv_mouse_mode != LV_MODE_DRAG && me.me_x < (int)(width - 2))) {
+        return false;
+    }
+
+    if (me.me_state == BUTTON_STATE_RELEASED) {
+        this->lv_mouse_y = -1;
+        this->lv_mouse_mode = LV_MODE_NONE;
+        return true;
+    }
+
+    int scroll_top, scroll_bottom, shift_amount = 0, new_top = 0;
+    double top_pct, bot_pct, pct;
+
+    top_pct = (double)this->get_top() / (double)inner_height;
+    bot_pct = (double)this->get_bottom() / (double)inner_height;
+    scroll_top = (this->get_y() + (int)(top_pct * (double)height));
+    scroll_bottom = (this->get_y() + (int)(bot_pct * (double)height));
+
+    if (this->lv_mouse_mode == LV_MODE_NONE) {
+        if (scroll_top <= me.me_y && me.me_y <= scroll_bottom) {
+            this->lv_mouse_mode        = LV_MODE_DRAG;
+            this->lv_mouse_y = me.me_y - scroll_top;
+        }
+        else if (me.me_y < scroll_top) {
+            this->lv_mouse_mode = LV_MODE_UP;
+        }
+        else {
+            this->lv_mouse_mode = LV_MODE_DOWN;
+        }
+    }
+
+    switch (this->lv_mouse_mode) {
+    case LV_MODE_NONE:
+        assert(0);
+        break;
+
+    case LV_MODE_UP:
+        if (me.me_y < scroll_top) {
+            shift_amount = -1 * height;
+        }
+        break;
+
+    case LV_MODE_DOWN:
+        if (me.me_y > scroll_bottom) {
+            shift_amount = height;
+        }
+        break;
+
+    case LV_MODE_DRAG:
+        pct     = (double)inner_height / (double)height;
+        new_top = me.me_y - this->get_y() - this->lv_mouse_y;
+        new_top = (int)floor(((double)new_top * pct) + 0.5);
+        this->set_top(vis_line_t(new_top));
+        break;
+    }
+
+    if (shift_amount != 0) {
+        this->shift_top(vis_line_t(shift_amount));
+    }
+
+    return true;
+}
