@@ -414,21 +414,25 @@ bool external_log_format::scan(std::vector<logline> &dst,
 {
     if (this->jlf_json) {
         auto_mem<yajl_handle_t> handle(yajl_free);
-        yajlpp_parse_context ypc("", json_log_handlers);
+        yajlpp_parse_context &ypc = *(this->jlf_parse_context);
         logline ll(offset, 0, 0, logline::LEVEL_INFO);
         json_log_userdata jlu;
         bool retval = false;
 
+        handle = yajl_alloc(&this->jlf_parse_context->ypc_callbacks,
+                            NULL,
+                            this->jlf_parse_context.get());
+        ypc.reset(json_log_handlers);
         ypc.ypc_userdata = &jlu;
         ypc.ypc_ignore_unused = true;
         ypc.ypc_alt_callbacks.yajl_start_array = json_array_start;
         ypc.ypc_alt_callbacks.yajl_start_map = json_array_start;
-        handle = yajl_alloc(&ypc.ypc_callbacks, NULL, &ypc); // XXX
         jlu.jlu_format = this;
         jlu.jlu_base_line = &ll;
         jlu.jlu_line_value = prefix;
         jlu.jlu_handle = handle;
-        if (yajl_parse(handle.in(), (const unsigned char *)prefix, len) == yajl_status_ok &&
+        if (yajl_parse(handle.in(),
+                       (const unsigned char *)prefix, len) == yajl_status_ok &&
             yajl_complete_parse(handle.in()) == yajl_status_ok) {
             for (int lpc = 0; lpc < jlu.jlu_sub_line_count; lpc++) {
                 ll.set_sub_offset(lpc);
@@ -573,7 +577,9 @@ void external_log_format::annotate(const std::string &line,
                          pi.get_substr(pc[vd.vd_index]),
                          vd.vd_identifier,
                          scaling,
-                         vd.vd_column));
+                         vd.vd_column,
+                         pc[vd.vd_index]->c_begin,
+                         pc[vd.vd_index]->c_end));
 
         if (pc[vd.vd_index]->c_begin != -1 && vd.vd_identifier) {
             lr.lr_start = pc[vd.vd_index]->c_begin;
@@ -646,26 +652,32 @@ void external_log_format::get_subline(const logline &ll,
 
     if (this->jlf_cached_offset != ll.get_offset()) {
         auto_mem<yajl_handle_t> handle(yajl_free);
-        yajlpp_parse_context ypc("", json_log_rewrite_handlers);
+        yajlpp_parse_context &ypc = *(this->jlf_parse_context);
         view_colors &vc = view_colors::singleton();
         json_log_userdata jlu;
 
-        jlu.jlu_format = this;
-        jlu.jlu_line = &ll;
-        ypc.ypc_userdata = &jlu;
-        ypc.ypc_ignore_unused = true;
         this->jlf_cached_line.clear();
         this->jlf_line_values.clear();
         this->jlf_line_offsets.clear();
         this->jlf_line_attrs.clear();
+
+        handle = yajl_alloc(&this->jlf_parse_context->ypc_callbacks,
+                            NULL,
+                            this->jlf_parse_context.get());
+        ypc.reset(json_log_rewrite_handlers);
+        ypc.ypc_userdata = &jlu;
+        ypc.ypc_ignore_unused = true;
         ypc.ypc_alt_callbacks.yajl_start_array = json_array_start;
         ypc.ypc_alt_callbacks.yajl_end_array = json_array_end;
         ypc.ypc_alt_callbacks.yajl_start_map = json_array_start;
         ypc.ypc_alt_callbacks.yajl_end_map = json_array_end;
-        handle = yajl_alloc(&ypc.ypc_callbacks, NULL, &ypc);
-        jlu.jlu_handle = handle.in();
+        jlu.jlu_format = this;
+        jlu.jlu_line = &ll;
+        jlu.jlu_handle = handle;
         jlu.jlu_line_value = line;
-        if (yajl_parse(handle.in(), (const unsigned char *)line, len) == yajl_status_ok &&
+
+        yajl_status parse_status = yajl_parse(handle.in(), (const unsigned char *)line, len);
+        if (parse_status == yajl_status_ok &&
             yajl_complete_parse(handle.in()) == yajl_status_ok) {
             std::vector<logline_value>::iterator lv_iter;
             std::vector<json_format_element>::iterator iter;
@@ -715,6 +727,7 @@ void external_log_format::get_subline(const logline &ll,
                         else if (lv_iter->lv_identifier) {
                             this->jlf_line_attrs[lr].insert(make_string_attr("style", vc.attrs_for_ident(str.c_str(), lr.length())));
                         }
+                        lv_iter->lv_origin = lr;
                         used_values[distance(this->jlf_line_values.begin(),
                                              lv_iter)] = true;
                     }
@@ -726,16 +739,18 @@ void external_log_format::get_subline(const logline &ll,
             }
             lines << endl;
             for (size_t lpc = 0; lpc < this->jlf_line_values.size(); lpc++) {
-                const logline_value &lv = this->jlf_line_values[lpc];
+                logline_value &lv = this->jlf_line_values[lpc];
 
                 if (used_values[lpc] ||
                     lv.lv_name == "body" ||
-                    lv.lv_name == this->elf_level_field)
+                    lv.lv_name == this->elf_level_field) {
                     continue;
+                }
 
                 const std::string str = lv.to_string();
                 size_t curr_pos = 0, nl_pos, line_len = -1;
 
+                lv.lv_origin.lr_start = lines.tellp();
                 do {
                     nl_pos = str.find('\n', curr_pos);
                     if (nl_pos != std::string::npos) {
@@ -750,6 +765,7 @@ void external_log_format::get_subline(const logline &ll,
                     line_len = -1;
                 } while (nl_pos != std::string::npos &&
                          nl_pos < str.size());
+                lv.lv_origin.lr_end = lines.tellp();
             }
             this->jlf_cached_line = lines.str();
 
@@ -829,6 +845,10 @@ void external_log_format::build(std::vector<std::string> &errors)
                              this->elf_name +
                              ": JSON logs cannot have regexes");
         }
+        if (this->jlf_json) {
+            this->jlf_parse_context.reset(new yajlpp_parse_context(this->elf_name));
+        }
+
     }
     else {
         if (this->elf_patterns.empty()) {
@@ -853,9 +873,22 @@ void external_log_format::build(std::vector<std::string> &errors)
     for (std::map<string, value_def>::iterator iter = this->elf_value_defs.begin();
          iter != this->elf_value_defs.end();
          ++iter) {
+        std::vector<std::string>::iterator act_iter;
+
         if (iter->second.vd_column == -1) {
             iter->second.vd_column = this->elf_column_count++;
         }
+
+        for (act_iter = iter->second.vd_action_list.begin();
+            act_iter != iter->second.vd_action_list.end();
+            ++act_iter) {
+            if (this->lf_action_defs.find(*act_iter) ==
+                this->lf_action_defs.end()) {
+                errors.push_back("error:" +
+                    this->elf_name + ":" + iter->first +
+                    ": cannot find action -- " + (*act_iter));
+            }
+        }
     }
 
     if (!this->jlf_json && this->elf_samples.empty()) {
