@@ -121,7 +121,9 @@ logfile_sub_source::logfile_sub_source()
     : lss_flags(0),
       lss_filter_generation(1),
       lss_filtered_count(0)
-{}
+{
+    this->clear_line_size_cache();
+}
 
 logfile_sub_source::~logfile_sub_source()
 { }
@@ -170,8 +172,6 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
 {
     content_line_t line(0);
 
-    size_t tab;
-
     assert(row >= 0);
     assert((size_t)row < this->lss_index.size());
 
@@ -189,10 +189,6 @@ void logfile_sub_source::text_value_for_line(textview_curses &tc,
     this->lss_token_value =
         this->lss_token_file->read_line(this->lss_token_line);
 
-    while ((tab = this->lss_token_value.find('\t')) != string::npos) {
-        this->lss_token_value = this->lss_token_value.replace(tab, 1, 8, ' ');
-    }
-
     this->lss_token_date_end = 0;
     value_out = this->lss_token_value;
     if (this->lss_flags & F_SCRUB) {
