@@ -450,8 +450,9 @@ static void load_time_bookmarks(void)
                     continue;
                 }
 
-                if (part_name == NULL)
+                if (part_name == NULL) {
                     continue;
+                }
 
                 if (!dts.scan(log_time, NULL, &log_tm, log_tv)) {
                     continue;
@@ -530,7 +531,7 @@ static void load_time_bookmarks(void)
 
         lss.find(lf->get_filename().c_str(), base_content_line);
 
-        fprintf(stderr, "checking bookmarks for %s\n", lf->get_filename().c_str());
+        fprintf(stderr, "checking time offsets for %s\n", lf->get_filename().c_str());
 
         logfile::iterator line_iter = lf->begin();
 
@@ -687,7 +688,7 @@ static int read_last_search(yajlpp_parse_context *ypc, const unsigned char *str,
                      ypc->get_path_fragment(-2));
     view_index = view_name - lnav_view_strings;
 
-    if (view_index < LNV__MAX) {
+    if (view_index < LNV__MAX && !regex.empty()) {
         execute_search((lnav_view_t)view_index, regex);
         lnav_data.ld_views[view_index].set_follow_search(false);
     }
@@ -715,6 +716,25 @@ static int read_top_line(yajlpp_parse_context *ypc, long long value)
     return 1;
 }
 
+static int read_word_wrap(yajlpp_parse_context *ypc, int value)
+{
+    const char **         view_name;
+    int view_index;
+
+    view_name = find(lnav_view_strings,
+                     lnav_view_strings + LNV__MAX,
+                     ypc->get_path_fragment(-2));
+    view_index = view_name - lnav_view_strings;
+    if (view_index < LNV__MAX) {
+        textview_curses &tc = lnav_data.ld_views[view_index];
+
+        fprintf(stderr, "set word wrap %d\n", value);
+        tc.set_word_wrap(value);
+    }
+
+    return 1;
+}
+
 static int read_commands(yajlpp_parse_context *ypc, const unsigned char *str, size_t len)
 {
     std::string cmdline = std::string((const char *)str, len);
@@ -725,11 +745,12 @@ static int read_commands(yajlpp_parse_context *ypc, const unsigned char *str, si
 }
 
 static struct json_path_handler view_info_handlers[] = {
-    json_path_handler("/save-time",              read_save_time),
-    json_path_handler("/files#",                 read_files),
-    json_path_handler("/views/([^.]+)/top_line", read_top_line),
-    json_path_handler("/views/([^.]+)/search",   read_last_search),
-    json_path_handler("/commands#",              read_commands),
+    json_path_handler("/save-time",               read_save_time),
+    json_path_handler("/files#",                  read_files),
+    json_path_handler("/views/([^/]+)/top_line",  read_top_line),
+    json_path_handler("/views/([^/]+)/search",    read_last_search),
+    json_path_handler("/views/([^/]+)/word_wrap", read_word_wrap),
+    json_path_handler("/commands#",               read_commands),
 
     json_path_handler()
 };
@@ -786,6 +807,7 @@ static void save_time_bookmarks(void)
     auto_mem<sqlite3_stmt> stmt(sqlite3_finalize);
 
     if (sqlite3_open(db_path.c_str(), db.out()) != SQLITE_OK) {
+        fprintf(stderr, "error: unable to open bookmark DB -- %s\n", db_path.c_str());
         return;
     }
 
@@ -813,7 +835,7 @@ static void save_time_bookmarks(void)
                            stmt.out(),
                            NULL) != SQLITE_OK) {
         fprintf(stderr,
-                "error: could not prepare bookmark replace statemnt -- %s\n",
+                "error: could not prepare bookmark delete statemnt -- %s\n",
                 sqlite3_errmsg(db));
         return;
     }
@@ -1065,7 +1087,7 @@ static void save_time_bookmarks(void)
         }
 
         if (sqlite3_bind_int64(stmt.in(), 6, offset.tv_usec) != SQLITE_OK) {
-            fprintf(stderr, "error: could not bind offset_sec -- %s\n",
+            fprintf(stderr, "error: could not bind offset_usec -- %s\n",
                     sqlite3_errmsg(db.in()));
             return;
         }
@@ -1127,7 +1149,7 @@ void save_session(void)
             yajlpp_map root_map(handle);
 
             root_map.gen("save-time");
-            root_map.gen(time(NULL));
+            root_map.gen((long long)time(NULL));
 
             root_map.gen("files");
 
@@ -1157,8 +1179,8 @@ void save_session(void)
                     view_map.gen("top_line");
 
                     tc.get_dimensions(height, width);
-                    if ((tc.get_top() + height) > tc.get_inner_height()) {
-                        view_map.gen(-1);
+                    if (tc.get_top() >= tc.get_top_for_last_row()) {
+                        view_map.gen(-1LL);
                     }
                     else{
                         view_map.gen((long long)tc.get_top());
@@ -1166,6 +1188,9 @@ void save_session(void)
 
                     view_map.gen("search");
                     view_map.gen(lnav_data.ld_last_search[lpc]);
+
+                    view_map.gen("word_wrap");
+                    view_map.gen(tc.get_word_wrap());
                 }
             }
 
